@@ -22,9 +22,14 @@
 #include <webkit/webkit.h>
 #include <JavaScriptCore/JavaScript.h>
 #include <glib/gi18n.h>
+#include <glib/gstdio.h>
 #include <string.h>
 #include <gio/gio.h>
 
+#if HAVE_UNISTD_H
+    #include <unistd.h>
+#endif
+
 struct _MidoriAddons
 {
     GtkVBox parent_instance;
@@ -212,14 +217,20 @@ _addons_get_directories (MidoriAddons* addons)
     {
         path = g_build_path (G_DIR_SEPARATOR_S, g_get_user_data_dir (),
                              PACKAGE_NAME, folders[i], NULL);
-        directories = g_slist_prepend (directories, path);
+        if (g_access (path, X_OK) == 0)
+            directories = g_slist_prepend (directories, path);
+        else
+            g_free (path);
 
         datadirs = g_get_system_data_dirs ();
         while (*datadirs)
         {
             path = g_build_path (G_DIR_SEPARATOR_S, *datadirs,
                                  PACKAGE_NAME, folders[i], NULL);
-            directories = g_slist_prepend (directories, path);
+            if (g_access (path, X_OK) == 0)
+                directories = g_slist_prepend (directories, path);
+            else
+                g_free (path);
             datadirs++;
         }
     }
@@ -796,8 +807,12 @@ midori_web_widget_context_ready_cb (GtkWidget*         web_widget,
     gchar* message;
 
     uri = katze_object_get_string (web_widget, "uri");
-    if (!uri)
+    /* Don't run scripts or styles on blank or special pages */
+    if (!(uri && *uri && strncmp (uri, "about:", 6)))
+    {
+        g_free (uri);
         return;
+    }
 
     elements = addons->elements;
     while (elements)
