@@ -1,5 +1,6 @@
 /*
  Copyright (C) 2008-2009 Christian Dywan <christian@twotoasts.de>
+ Copyright (C) 2009 Alexander Butenko <a.butenka@gmail.com>
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -13,16 +14,31 @@
     #include <config.h>
 #endif
 
-#include "compat.h"
 #include "sokoke.h"
 
 #define SM "http://www.searchmash.com/search/"
 
+static void
+sokoke_assert_str_equal (const gchar* input,
+                         const gchar* result,
+                         const gchar* expected)
+{
+    if (g_strcmp0 (result, expected))
+    {
+        g_error ("Input: %s\nExpected: %s\nResult: %s",
+                 input ? input : "NULL",
+                 expected ? expected : "NULL",
+                 result ? result : "NULL");
+    }
+}
+
 static void
 test_input (const gchar* input,
             const gchar* expected)
 {
     static KatzeArray* search_engines = NULL;
+    gchar* uri;
+
     if (G_UNLIKELY (!search_engines))
     {
         KatzeItem* item;
@@ -40,22 +56,37 @@ test_input (const gchar* input,
         g_object_unref (item);
     }
 
-    gchar* uri = sokoke_magic_uri (input, search_engines);
-    if (g_strcmp0 (uri, expected))
+    uri = sokoke_magic_uri (input);
+    if (!uri)
     {
-        g_error ("Input: %s\nExpected: %s\nResult: %s",
-                 input ? input : "NULL",
-                 expected ? expected : "NULL",
-                 uri ? uri : "NULL");
+        gchar** parts;
+        gchar* keywords = NULL;
+        const gchar* search_uri = NULL;
+
+        /* Do we have a keyword and a string? */
+        parts = g_strsplit (input, " ", 2);
+        if (parts[0])
+        {
+            KatzeItem* item;
+            if ((item = katze_array_find_token (search_engines, parts[0])))
+            {
+                keywords = g_strdup (parts[1] ? parts[1] : "");
+                search_uri = katze_item_get_uri (item);
+            }
+        }
+        g_strfreev (parts);
+
+        uri = keywords ? sokoke_search_uri (search_uri, keywords) : NULL;
+
+        g_free (keywords);
     }
+    sokoke_assert_str_equal (input, uri, expected);
     g_free (uri);
 }
 
 static void
 magic_uri_uri (void)
 {
-    gchar* a, *b;
-
     test_input ("ftp://ftp.mozilla.org", "ftp://ftp.mozilla.org");
     test_input ("ftp://ftp.mozilla.org/pub", "ftp://ftp.mozilla.org/pub");
     test_input ("http://www.example.com", "http://www.example.com");
@@ -64,14 +95,10 @@ magic_uri_uri (void)
     test_input ("example.com", "http://example.com");
     test_input ("www.google..com", "http://www.google..com");
     test_input ("/home/user/midori.html", "file:///home/user/midori.html");
-    a = g_get_current_dir ();
-    b = g_strconcat ("file://", a, G_DIR_SEPARATOR_S, "magic-uri.c", NULL);
-    g_free (a);
-    test_input ("magic-uri.c", b);
-    g_free (b);
     test_input ("localhost", "http://localhost");
     test_input ("localhost:8000", "http://localhost:8000");
     test_input ("localhost/rss", "http://localhost/rss");
+    test_input ("10.0.0.1", "http://10.0.0.1");
     test_input ("192.168.1.1", "http://192.168.1.1");
     test_input ("192.168.1.1:8000", "http://192.168.1.1:8000");
     test_input ("file:///home/mark/foo/bar.html",
@@ -84,27 +111,47 @@ magic_uri_uri (void)
 static void
 magic_uri_idn (void)
 {
-    #if HAVE_LIBIDN
-    test_input ("http://www.münchhausen.at", "http://www.xn--mnchhausen-9db.at");
-    test_input ("http://www.خداوند.com/", "http://www.xn--mgbndb8il.com/");
-    test_input ("айкидо.com", "http://xn--80aildf0a.com");
-    test_input ("http://東京理科大学.jp", "http://xn--1lq68wkwbj6ugkpigi.jp");
-    test_input ("https://青のネコ",  "https://xn--u9jthzcs263c");
+    typedef struct
+    {
+        const gchar* before;
+        const gchar* after;
+    } URIItem;
+
+    static const URIItem items[] = {
+    #if HAVE_LIBIDN || defined (HAVE_LIBSOUP_2_27_90)
+     { "http://www.münchhausen.at", "http://www.xn--mnchhausen-9db.at" },
+     { "http://www.خداوند.com/", "http://www.xn--mgbndb8il.com/" },
+     { "айкидо.com", "xn--80aildf0a.com" },
+     { "http://東京理科大学.jp", "http://xn--1lq68wkwbj6ugkpigi.jp" },
+     { "https://青のネコ",  "https://xn--u9jthzcs263c" },
     #else
-    test_input ("http://www.münchhausen.at", "http://www.münchhausen.at");
-    test_input ("http://www.خداوند.com/", "http://www.خداوند.com/");
+     { "http://www.münchhausen.at", NULL },
+     { "http://www.خداوند.com/", NULL },
+     { "айкидо.com", NULL },
+     { "http://東京理科大学.jp", NULL },
+     { "https://青のネコ.co.jp",  NULL },
+    #endif
+    { "http://en.wikipedia.org/wiki/Kölsch_language", NULL },
+    { "file:///home/mark/frühstück", NULL },
+    { "about:version", NULL },
+     };
+    guint i;
+
+    for (i = 0; i < G_N_ELEMENTS (items); i++)
+    {
+        gchar* result = sokoke_uri_to_ascii (items[i].before);
+        const gchar* after = items[i].after ? items[i].after : items[i].before;
+        sokoke_assert_str_equal (items[i].before, result, after);
+        g_free (result);
+    }
+
+    #if HAVE_LIBIDN
+    test_input ("айкидо.com", "http://xn--80aildf0a.com");
+    #else
     test_input ("айкидо.com", "http://айкидо.com");
-    test_input ("http://東京理科大学.jp", "http://東京理科大学.jp");
-    test_input ("https://青のネコ.co.jp",  "https://青のネコ.co.jp");
     #endif
-    test_input ("http://en.wikipedia.org/wiki/Kölsch_language",
-                "http://en.wikipedia.org/wiki/Kölsch_language");
-    test_input ("en.wikipedia.org/wiki/Kölsch_language",
-                "http://en.wikipedia.org/wiki/Kölsch_language");
     test_input ("sm Küchenzubehör", SM "Küchenzubehör");
     test_input ("sm 東京理科大学", SM "東京理科大学");
-    test_input ("file:///home/mark/frühstück",
-                "file:///home/mark/frühstück");
 }
 
 static void
@@ -120,6 +167,8 @@ magic_uri_search (void)
     test_input ("gtk2.0", NULL);
     test_input ("pcre++", NULL);
     test_input ("sm pcre++", SM "pcre%2B%2B");
+    test_input ("5580", NULL);
+    test_input ("sm 5580", SM "5580");
     test_input ("midori0.1.0", NULL);
     test_input ("midori 0.1.0", NULL);
     test_input ("search:cats", NULL);
@@ -142,6 +191,7 @@ magic_uri_search (void)
     test_input ("g max@mustermann.de", NULL);
     test_input ("g inurl:http://twotoasts.de bug", NULL);
     test_input ("sm", SM);
+    /* test_input ("LT_PREREQ(2.2)", NULL); */
 }
 
 static void
@@ -171,10 +221,61 @@ magic_uri_performance (void)
     g_print ("\nTime needed for URI tests: %f ", g_test_timer_elapsed ());
 }
 
+static void
+magic_uri_format (void)
+{
+    typedef struct
+    {
+        const gchar* before;
+        const gchar* after;
+    } URIItem;
+
+    static const URIItem items[] = {
+     { "http://www.csszengarden.com", NULL },
+     { "http://live.gnome.org/GTK+/3.0/Tasks", NULL },
+     { "http://www.johannkönig.com/index.php?ausw=home", NULL },
+     { "http://digilife.bz/wiki/index.php?Python%E3%81%AE%E9%96%8B%E7%99%BA%E6%89%8B%E9%A0%86",
+       "http://digilife.bz/wiki/index.php?Pythonの開発手順" },
+     { "http://die-welt.net/~evgeni/LenovoBatteryLinux/", NULL },
+     { "http://wiki.c3sl.ufpr.br/multiseat/index.php/Xephyr_Solution", NULL },
+     { "http://şøñđëřżēıċħęŋđőmæîņĭśŧşũþėŗ.de/char.jpg", NULL },
+     { "http://www.ⓖⓝⓞⓜⓔ.org/", "http://www.gnome.org/" },
+     };
+    guint i;
+
+    for (i = 0; i < G_N_ELEMENTS (items); i++)
+    {
+        gchar* result = sokoke_format_uri_for_display (items[i].before);
+        const gchar* after = items[i].after ? items[i].after : items[i].before;
+        sokoke_assert_str_equal (items[i].before, result, after);
+        g_free (result);
+    }
+}
+
+static void
+magic_uri_prefetch (void)
+{
+    g_assert (!sokoke_prefetch_uri (NULL));
+    g_assert (sokoke_prefetch_uri ("http://google.com"));
+    g_assert (sokoke_prefetch_uri ("http://google.com"));
+    g_assert (sokoke_prefetch_uri ("http://googlecom"));
+    g_assert (sokoke_prefetch_uri ("http://1kino.com"));
+    g_assert (sokoke_prefetch_uri ("http://"));
+    g_assert (!sokoke_prefetch_uri ("http:/"));
+    g_assert (!sokoke_prefetch_uri ("http"));
+    g_assert (!sokoke_prefetch_uri ("ftp://ftphost.org"));
+    g_assert (!sokoke_prefetch_uri ("http://10.0.0.1"));
+    g_assert (!sokoke_prefetch_uri ("about:blank"));
+    g_assert (!sokoke_prefetch_uri ("javascript: alert()"));
+}
+
 int
 main (int    argc,
       char** argv)
 {
+    /* libSoup uses threads, therefore if WebKit is built with libSoup
+       or Midori is using it, we need to initialize threads. */
+    if (!g_thread_supported ()) g_thread_init (NULL);
     g_test_init (&argc, &argv, NULL);
     gtk_init_check (&argc, &argv);
 
@@ -183,6 +284,8 @@ main (int    argc,
     g_test_add_func ("/magic-uri/search", magic_uri_search);
     g_test_add_func ("/magic-uri/pseudo", magic_uri_pseudo);
     g_test_add_func ("/magic-uri/performance", magic_uri_performance);
+    g_test_add_func ("/magic-uri/format", magic_uri_format);
+    g_test_add_func ("/magic-uri/prefetch", magic_uri_prefetch);
 
     return g_test_run ();
 }
